## Copyright © LIMSI-CNRS (2013)
##
## contributor(s) : Jorge Gascon, Damien Touraine, David Poirier-Quinot,
## Laurent Pointal, Julian Adenauer, 
## 
## This software is a computer program whose purpose is to distribute
## blender to render on CAVE(TM) device systems.
## 
## This software is governed by the CeCILL  license under French law and
## abiding by the rules of distribution of free software.  You can  use, 
## modify and/ or redistribute the software under the terms of the CeCILL
## license as circulated by CEA, CNRS and INRIA at the following URL
## "http://www.cecill.info". 
## 
## As a counterpart to the access to the source code and  rights to copy,
## modify and redistribute granted by the license, users are provided only
## with a limited warranty  and the software's author,  the holder of the
## economic rights,  and the successive licensors  have only  limited
## liability. 
## 
## In this respect, the user's attention is drawn to the risks associated
## with loading,  using,  modifying and/or developing or reproducing the
## software by the user in light of its specific status of free software,
## that may mean  that it is complicated to manipulate,  and  that  also
## therefore means  that it is reserved for developers  and  experienced
## professionals having in-depth computer knowledge. Users are therefore
## encouraged to load and test the software's suitability as regards their
## requirements in conditions enabling the security of their systems and/or 
## data to be ensured and,  more generally, to use and operate it in the 
## same conditions as regards security. 
## 
## The fact that you are presently reading this means that you have had
## knowledge of the CeCILL license and that you accept its terms.
## 

import sys
import xml.sax.handler
import optparse
import os
import blender_cave.base
import blender_cave.exceptions

class Configure(blender_cave.base.Base, xml.sax.handler.ContentHandler, xml.sax.handler.EntityResolver):

    def __init__(self, parent, config_file, config_paths):
        super(Configure, self).__init__(parent)

        self._main         = self
        if type(config_paths) is str:
            self._config_paths = config_paths.split(os.pathsep)
        else:
            self._config_paths = config_paths

        self._parser = xml.sax.make_parser()
        self._parser.setEntityResolver(self)

        from . import root
        self._root = root.XML(self)
        self._parser.setContentHandler(self._root)

        self._parser.parse(self._lookForConfigurationFile(config_file))

    def resolveEntity(self,publicID,systemID):
        evaluated = self._root._evaluateInput(systemID)
        filename  = self._lookForConfigurationFile(evaluated)
        if filename is None:
            if evaluated != systemID:
                name = systemID + ' (' + evaluated + ')'
            else:
                name = systemID
            raise blender_cave.exceptions.Configure('Cannot load ' + name + ' configuration file')
        return filename

    def _lookForConfigurationFile(self, filename):
        if filename is not None:
            for path in self._config_paths:
                complete_filename = os.path.join(path, filename)
                try:
                    file = open(complete_filename)
                except:
                    continue
                return complete_filename
        return None

    def getConfiguration(self):
        return self._root.getConfiguration()

    def getScreenConfiguration(self, configuration, name, config = None):
        result = {}
        if ('screens' in configuration) and (name in configuration['screens']):
            result['screen'] = configuration['screens'][name]
        else:
            result['screen'] = None

        if (result['screen'] is not None) and ('computer' in result['screen']):
            result['computer_name'] = result['screen']['computer']
        else:
            result['computer_name'] = None

        if ('computers' in configuration) and (result['computer_name'] in configuration['computers']):
            result['computer'] = configuration['computers'][result['computer_name']]
        else:
            result['computer'] = None

        if config is not None:
            for element in ['port', 'users', 'processor']:
                if element in configuration:
                    result[element] = configuration[element]

            current_config = configuration['starter']['configs'][config]
            result['number_screens'] = len(current_config)
            result['screen_id'] = current_config.index(name)
            master = current_config[0]
            result['is_master'] = (master == name)
            master_computer = configuration['screens'][master]['computer']
            result['master_host'] = configuration['computers'][master_computer]['hostname']

            if 'focus_master' in configuration and configuration['focus_master'] and result['is_master']:
                result['screen']['focus'] = True
            else:
                result['screen']['focus'] = False



        return result
