# -*- coding: iso-8859-1 -*-
## Copyright  LIMSI-CNRS (2013)
##
## contributor(s) : Jorge Gascon, Damien Touraine, David Poirier-Quinot,
## Laurent Pointal, Julian Adenauer, 
## 
## This software is a computer program whose purpose is to distribute
## blender to render on CAVE(TM) device systems.
## 
## This software is governed by the CeCILL  license under French law and
## abiding by the rules of distribution of free software.  You can  use, 
## modify and/ or redistribute the software under the terms of the CeCILL
## license as circulated by CEA, CNRS and INRIA at the following URL
## "http://www.cecill.info". 
## 
## As a counterpart to the access to the source code and  rights to copy,
## modify and redistribute granted by the license, users are provided only
## with a limited warranty  and the software's author,  the holder of the
## economic rights,  and the successive licensors  have only  limited
## liability. 
## 
## In this respect, the user's attention is drawn to the risks associated
## with loading,  using,  modifying and/or developing or reproducing the
## software by the user in light of its specific status of free software,
## that may mean  that it is complicated to manipulate,  and  that  also
## therefore means  that it is reserved for developers  and  experienced
## professionals having in-depth computer knowledge. Users are therefore
## encouraged to load and test the software's suitability as regards their
## requirements in conditions enabling the security of their systems and/or 
## data to be ensured and,  more generally, to use and operate it in the 
## same conditions as regards security. 
## 
## The fact that you are presently reading this means that you have had
## knowledge of the CeCILL license and that you accept its terms.
## 

import os
import sys
import stat
import argparse
import blender_cave.exceptions
import blender_cave.base

class _Parser(argparse.ArgumentParser):

    def __init__(self):
        super(_Parser, self).__init__()
        self._errors = []

    def error(self, msg):
        self._errors.append(msg)
        pass

    def getErrors(self):
        return self._errors

class Environment(blender_cave.base.Base):

    def __init__(self, parent, specific_options_method = None):
        super(Environment, self).__init__(parent)

        try:
            arguments = sys.argv[(sys.argv.index('-')+1):]
        except ValueError:
            try:
                arguments = sys.argv[(sys.argv.index('--')+1):]
            except ValueError:
                arguments = sys.argv[1:]

        parser = _Parser()
        parser.add_argument("--config",       dest="config")
        parser.add_argument("--screen",       dest="screen")
        parser.add_argument("--config-file",  dest="config_file")
        parser.add_argument("--config-paths", dest="config_paths")
        parser.add_argument("-b",             dest="background",   action="store_true", default=False)

        if specific_options_method is not None:
            specific_options_method(parser)

        self._environment = parser.parse_args(arguments)

        parsing_errors = parser.getErrors()
        for parsing_error in parsing_errors:
            self.getLogger().warning('Parsing error : "' + str(parsing_error) + '"')

    def getEnvironment(self, key):
        if hasattr(self._environment, key):
            return getattr(self._environment, key)
        return None
