# -*- coding: iso-8859-1 -*-
## Copyright  LIMSI-CNRS (2013)
##
## contributor(s) : Jorge Gascon, Damien Touraine, David Poirier-Quinot,
## Laurent Pointal, Julian Adenauer, 
## 
## This software is a computer program whose purpose is to distribute
## blender to render on CAVE(TM) device systems.
## 
## This software is governed by the CeCILL  license under French law and
## abiding by the rules of distribution of free software.  You can  use, 
## modify and/ or redistribute the software under the terms of the CeCILL
## license as circulated by CEA, CNRS and INRIA at the following URL
## "http://www.cecill.info". 
## 
## As a counterpart to the access to the source code and  rights to copy,
## modify and redistribute granted by the license, users are provided only
## with a limited warranty  and the software's author,  the holder of the
## economic rights,  and the successive licensors  have only  limited
## liability. 
## 
## In this respect, the user's attention is drawn to the risks associated
## with loading,  using,  modifying and/or developing or reproducing the
## software by the user in light of its specific status of free software,
## that may mean  that it is complicated to manipulate,  and  that  also
## therefore means  that it is reserved for developers  and  experienced
## professionals having in-depth computer knowledge. Users are therefore
## encouraged to load and test the software's suitability as regards their
## requirements in conditions enabling the security of their systems and/or 
## data to be ensured and,  more generally, to use and operate it in the 
## same conditions as regards security. 
## 
## The fact that you are presently reading this means that you have had
## knowledge of the CeCILL license and that you accept its terms.
## 

import os
import logging
from types import MethodType

def getInformation(configuration):
    try:
        computer_name = configuration['computer']['name']
        screen_name   = configuration['screen']['name']
        log_node      = configuration['computer']['system']['log']
        log_path      = log_node['path']
        verbosity     = log_node['verbosity']
    except KeyError:
        return (None, None)
    log_file = os.path.join(log_path, computer_name + '_' + screen_name + '.log')
    return (log_file, verbosity)
    

class Logger():

    def __init__(self):
        self._verbosities = ['debug', 'info', 'warning', 'error', 'critical']

        for verbosity in self._verbosities:
            setattr(self, '_' + verbosity, [])
            setattr(self, verbosity, MethodType(self._process, verbosity))

        self._logger = None

    def setDestination(self, log_file, verbosity):

        self._logger = logging.getLogger('Blender CAVE')

        if log_file is None:
            self._handler = logging.StreamHandler()
            self._handler.setFormatter(logging.Formatter('[Blender CAVE] %(levelname)s: %(message)s'))
        else:
            self._handler = logging.FileHandler(log_file)
            self._handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)s: %(message)s'))

        self._logger.addHandler(self._handler)

        self._logger.setLevel(self._getVerbosity(verbosity))

        for verbosity in self._verbosities:
            for message in getattr(self, '_' + verbosity):
                getattr(self._logger, verbosity)(message)
            setattr(self, '_' + verbosity, None)

    def log_traceback(self, error):
        if error:
            method = self.error
        else:
            method = self.warning
        import traceback
        method('***************************')
        method(traceback.format_exc())
        method('***************************')

    def forceLog(self):
        if self._logger == None:
            self.setDestination(None, None)

    def _process(self, verbosity, *messages, sep=' '):
        elements = []
        for message in messages:
            elements.append(str(message))
        if self._logger is None:
            for message in (sep.join(elements)).split('\n'):
                getattr(self, '_' + verbosity).append(message)
        else:
            for message in (sep.join(elements)).split('\n'):
                getattr(self._logger, verbosity)(message)

    def _getVerbosity(self, verbosity):
        if verbosity is not None:
            verbosity = verbosity.lower()

        if verbosity == 'debug':
            return logging.DEBUG
        if verbosity == 'info':
            return logging.INFO
        if verbosity == 'warning':
            return logging.WARNING
        if verbosity == 'error':
            return logging.ERROR
        if verbosity == 'critical':
            return logging.CRITICAL

        return logging.WARNING
