## Copyright © LIMSI-CNRS (2013)
##
## contributor(s) : Jorge Gascon, Damien Touraine, David Poirier-Quinot,
## Laurent Pointal, Julian Adenauer, 
## 
## This software is a computer program whose purpose is to distribute
## blender to render on CAVE(TM) device systems.
## 
## This software is governed by the CeCILL  license under French law and
## abiding by the rules of distribution of free software.  You can  use, 
## modify and/ or redistribute the software under the terms of the CeCILL
## license as circulated by CEA, CNRS and INRIA at the following URL
## "http://www.cecill.info". 
## 
## As a counterpart to the access to the source code and  rights to copy,
## modify and redistribute granted by the license, users are provided only
## with a limited warranty  and the software's author,  the holder of the
## economic rights,  and the successive licensors  have only  limited
## liability. 
## 
## In this respect, the user's attention is drawn to the risks associated
## with loading,  using,  modifying and/or developing or reproducing the
## software by the user in light of its specific status of free software,
## that may mean  that it is complicated to manipulate,  and  that  also
## therefore means  that it is reserved for developers  and  experienced
## professionals having in-depth computer knowledge. Users are therefore
## encouraged to load and test the software's suitability as regards their
## requirements in conditions enabling the security of their systems and/or 
## data to be ensured and,  more generally, to use and operate it in the 
## same conditions as regards security. 
## 
## The fact that you are presently reading this means that you have had
## knowledge of the CeCILL license and that you accept its terms.
## 

import blender_cave.configure.reusable

try:
    import mathutils
except ImportError:
    pass

class XML(blender_cave.configure.reusable.XML):

    def __init__(self, parent, name, attrs):
        super(XML, self).__init__(parent, name, attrs)
        self._attribute_list += ['pre', 'post']
        self._pre             = None
        self._post            = None

    def _addM(self, when, operation, value):
        if 'angle' in value:
            operation = mathutils.Matrix.Rotation(value['angle'], 4, value['vector'])
        else:
            operation = mathutils.Matrix.Translation(value['vector'])
        if getattr(self, '_' + when) is not None:
            new = operation * getattr(self, '_' + when)
        else:
            new = operation
        setattr(self, '_' + when, new)

    def _addS(self, when, operation, value):
        value = {operation : value}
        if getattr(self, '_' + when) is None:
            setattr(self, '_' + when, [value])
        else:
            getattr(self, '_' + when).append(value)


    def _getChildren(self, name, attrs):
        type = name.split('_')
        if len(type) == 2:
            if (type[0] == 'pre' or type[0] == 'post') and (type[1] == 'translation' or type[1] == 'rotation'):
                try:
                    vector = self.getVector(attrs, 3, 0.0)
                    value = {'vector' : vector}
                    if type[1] == 'rotation':
                        value['angle'] = self._getNumber(attrs['angle'])
                    if 'mathutils' in globals():
                        self._addM(type[0], type[1], value)
                    else:
                        self._addS(type[0], type[1], value)
                except KeyError:
                    pass
        return super(XML, self)._getChildren(name, attrs)

    def getConfiguration(self):
        self._updateFromParents()
        return super(XML, self).getConfiguration()
