import blender_cave
import bge

def initInstrument(instrument,soundFile):
    MUTE = True
    print('==', instrument, '==')
    print('- sound file:',soundFile)
    print('- position:', [round(elmt, 2) for elmt in instrument.worldPosition[::]])
    print('- state:', MUTE)

    osc_instrument = blender_cave.getOSC().getObject(instrument)
    osc_instrument.sound(soundFile)
    osc_instrument.loop(True)
    osc_instrument.start(True)
    osc_instrument.volume('%10')
    osc_instrument.mute(MUTE) # complety desactivate object (receive msgs no more)

def coupleObj2Usr(obj,osc_user):
    osc_obj = blender_cave.getOSC().getObject(obj)
    osc_object_user = blender_cave.getOSC().getObjectUser(osc_obj, osc_user)
    osc_object_user.mute(False)
    return osc_object_user

def main(controller):
	if not hasattr(bge.logic,'soundList'): # run only once
		print("----------.py Started ----------------------------")

		## bge.logic variables definition
		bge.logic.soundList = ['GuitarSolo.wav', 'BasseCountry.wav', 'Basse.wav', 'Bongo.wav', 'shaker.wav','micro1','micro2','pink_noise','white_noise']
		print("Available sounds (seeked by sfplay in Max OSC client):")
		for elmt in bge.logic.soundList: print(elmt)
		print('')
		bge.logic.instrumentList = []
		bge.logic.counter = 0

		# -- Rewrite MaxMSP config (already done through the .xml file)
		# bge.logic.configCount = 0
		# bge.logic.configList = ['binaural LSE PERS','ambisonic SPAT EVE','binaural SPAT EVE']
		# currentOSC_config = bge.logic.configList[bge.logic.configCount]
		# OSC.getGlobal().configuration(currentOSC_config)

		## Get ordered instruments (KX_GameObjects) list:
		scene = bge.logic.getCurrentScene()
		instrumentNameList = []
		for object in scene.objects:
			if 'Inst' in object.name: # just a convention: I named "Instrument.." the scene objects that will produce sound
				instrumentNameList.append(object.name)
				object.setVisible(False)
		instrumentNameList.sort()
		for instrumentName in instrumentNameList:
			bge.logic.instrumentList.append(scene.objects[instrumentName])
		print('Available Instruments (objects in blender):')
		for elmt in bge.logic.instrumentList: print(elmt)
		print('')

		## OSC Global Methods
		OSC = blender_cave.getOSC()
		# for elmt in dir(blender_cave): print (elmt)
		OSC.getGlobal().start(True)
		OSC.getGlobal().volume('%50')
		OSC.getGlobal().mute(False)
		# OSC.getGlobal().reset() # Automatically invoked when calling blender_cave.quit('')

		## OSC Object Methods
		for i in range(0,len(bge.logic.instrumentList)):
			initInstrument(bge.logic.instrumentList[i],bge.logic.soundList[i])
		print('')

		## OSC objectUser Methods
		userAmbisonic = OSC.getUser('Ambisonic')
		userBinaural1 = OSC.getUser('Binaural 1')
		userBinaural2 = OSC.getUser('Binaural 2')
		userStereo = OSC.getUser('Stereo')

		## Sound Equalizer
		blender_cave.getOSC().getObject(bge.logic.instrumentList[2]).volume('%80')

		try:
			coupleObj2Usr(bge.logic.instrumentList[0],userAmbisonic)
			coupleObj2Usr(bge.logic.instrumentList[1],userBinaural1)
			coupleObj2Usr(bge.logic.instrumentList[2],userBinaural2)
			coupleObj2Usr(bge.logic.instrumentList[3],userStereo)
			coupleObj2Usr(bge.logic.instrumentList[4],userAmbisonic)
			coupleObj2Usr(bge.logic.instrumentList[4],userBinaural1)
			coupleObj2Usr(bge.logic.instrumentList[4],userBinaural2)
			coupleObj2Usr(bge.logic.instrumentList[4],userStereo)
		except IndexError:
			print('no Instruments found in scene (active layer ... in .blend)')
		## OSC User Methods
		userAmbisonic.volume('%10')
		userAmbisonic.hrtf(4)
		userAmbisonic.presence(50)
		userAmbisonic.reverb_volume(50)
		userAmbisonic.warmth(50)
		userAmbisonic.brightness(50)

