import blender_cave.processor
import blender_cave
import bge
import mathutils
import math
import sys
import copy
import os

# sys.path.append('./../../scripts/')
from change_texture import setTexture

class Configure(blender_cave.processor.Configure):
    def __init__(self, parent, attrs):
        super(Configure, self).__init__(parent, attrs)

class Processor(blender_cave.processor.Processor):
    def __init__(self, parent, configuration):
        super(Processor, self).__init__(parent, configuration)

        self._scene = bge.logic.getCurrentScene()
        self._tv = self._scene.objects['Screen']
        self._tv_tex = "./standby_A.png"
        self._tv_mat  = "screen"
        self._to_change_texture = False
        # self.getLogger().debug(self._scene.objects)

        # To propagate every bge.logic item to the slave nodes (e.g. keyboard inputs)
        # (the master blenderplayer window must be active)
        if self.getBlenderCave().isMaster():
            self.getBlenderCave().getSceneSynchronizer().getItem(bge.logic).activate(True, True)


    def console(self, info):
        try:
            # Quit
            if info['key'] == 113:
                self.getBlenderCave().quit("pressed 'q' key")

            # Change Texture
            if info['state'] == 1 and info['key'] in [13,32]: # (triggered once)
                if info['key'] == 13: self._tv_tex = './A_1.png' # enter
                if info['key'] == 32: self._tv_tex = './A_2.png' # space
                setTexture(self._tv, self._tv_mat, self._tv_tex)
                self._to_change_texture = True
                self.getLogger().debug(self._tv_tex)

        except KeyError:
            pass


    def getSynchronizerBuffer(self):
        """For the master"""
        buffer = blender_cave.buffer.Buffer()
        self.getLogger().debug("## & ##")
        if self._to_change_texture:
            buffer.string(self._tv_tex)
            self._to_change_texture = False
            self.getLogger().debug("--> Master Load buffer:", self._tv_tex)

        return buffer

    def processSynchronizerBuffer(self, buffer):
        """For the slaves"""
        while not buffer.isEmpty():
            tex = buffer.string()
            setTexture(self._tv, "screen", tex)
            self.getLogger().debug("--> Slave read buffer:", tex)
