## Copyright © LIMSI-CNRS (2013)
##
## contributor(s) : Jorge Gascon, Damien Touraine, David Poirier-Quinot,
## Laurent Pointal, Julian Adenauer, 
## 
## This software is a computer program whose purpose is to distribute
## blender to render on CAVE(TM) device systems.
## 
## This software is governed by the CeCILL  license under French law and
## abiding by the rules of distribution of free software.  You can  use, 
## modify and/ or redistribute the software under the terms of the CeCILL
## license as circulated by CEA, CNRS and INRIA at the following URL
## "http://www.cecill.info". 
## 
## As a counterpart to the access to the source code and  rights to copy,
## modify and redistribute granted by the license, users are provided only
## with a limited warranty  and the software's author,  the holder of the
## economic rights,  and the successive licensors  have only  limited
## liability. 
## 
## In this respect, the user's attention is drawn to the risks associated
## with loading,  using,  modifying and/or developing or reproducing the
## software by the user in light of its specific status of free software,
## that may mean  that it is complicated to manipulate,  and  that  also
## therefore means  that it is reserved for developers  and  experienced
## professionals having in-depth computer knowledge. Users are therefore
## encouraged to load and test the software's suitability as regards their
## requirements in conditions enabling the security of their systems and/or 
## data to be ensured and,  more generally, to use and operate it in the 
## same conditions as regards security. 
## 
## The fact that you are presently reading this means that you have had
## knowledge of the CeCILL license and that you accept its terms.
## 

import sys
import os
import py_compile
import compileall

class Fake:
    def __init__(self):
        from blender_cave import logger
        self._logger = logger.Logger()

    def log_traceback(self, error = False):
        self._logger.log_traceback(error)
        if error:
            self._logger.forceLog()
            sys.exit()

    def setLoggerDestination(self, destination, verbosity):
        self._logger.setDesination(destination, verbosity)

    def getLogger(self):
        return self._logger

class Launcher:
    def __init__(self, config_paths, config_file):

        # Root can be obtained from current file !
        root = os.path.dirname(os.path.dirname(__file__))

        # Add main blender cave path ...
        sys.path.append(root)

        # ... compile it
        compileall.compile_dir(os.path.join(root, 'blender_cave'), quiet = True)

        fake = Fake()

        if (config_paths == None) or (config_file == None):
            from blender_cave import environment
            self._environnment = environment.Environment(fake, self._addParsingOptions)

            config_file   = self._environnment.getEnvironment('config_file')
            config_paths  = self._environnment.getEnvironment('config_paths')
            

        try:
            from blender_cave import configure
            from blender_cave import exceptions
            try:
                self._configurator  = configure.Configure(fake, config_file, config_paths)
            except exceptions.Configure as error:
                fake.getLogger().error(error)
                sys.exit()
            except:
                import traceback
                fake.getLogger().error(traceback.format_exc())
                sys.exit()
        except:
            fake.log_traceback(True)

    def getConfiguration(self):
        return self._configurator.getConfiguration()

    def getScreenConfiguration(self, configuration, name, config = None):
        return self._configurator.getScreenConfiguration(configuration, name, config)

    def _addParsingOptions(self, parser):
        parser.add_argument("--blend-file", dest="blend_file")
        parser.add_argument("-s",           dest="status",  action="store_true", default=False)
        parser.add_argument("-l",           dest="log_run", action="store_true", default=False)
