#!/usr/bin/env python3.2
## Copyright © LIMSI-CNRS (2013)
##
## contributor(s) : Jorge Gascon, Damien Touraine, David Poirier-Quinot,
## Laurent Pointal, Julian Adenauer,
##
## This software is a computer program whose purpose is to distribute
## blender to render on CAVE(TM) device systems.
##
## This software is governed by the CeCILL  license under French law and
## abiding by the rules of distribution of free software.  You can  use,
## modify and/ or redistribute the software under the terms of the CeCILL
## license as circulated by CEA, CNRS and INRIA at the following URL
## "http://www.cecill.info".
##
## As a counterpart to the access to the source code and  rights to copy,
## modify and redistribute granted by the license, users are provided only
## with a limited warranty  and the software's author,  the holder of the
## economic rights,  and the successive licensors  have only  limited
## liability.
##
## In this respect, the user's attention is drawn to the risks associated
## with loading,  using,  modifying and/or developing or reproducing the
## software by the user in light of its specific status of free software,
## that may mean  that it is complicated to manipulate,  and  that  also
## therefore means  that it is reserved for developers  and  experienced
## professionals having in-depth computer knowledge. Users are therefore
## encouraged to load and test the software's suitability as regards their
## requirements in conditions enabling the security of their systems and/or
## data to be ensured and,  more generally, to use and operate it in the
## same conditions as regards security.
##
## The fact that you are presently reading this means that you have had
## knowledge of the CeCILL license and that you accept its terms.
##

import optparse
import sys
import os
import socket
import subprocess
import signal

root=os.path.dirname(os.path.dirname(__file__))

import launcher
configurator = launcher.Launcher(None, None)

options  = configurator._environnment

screen_name       = options.getEnvironment('screen')
all_configuration = configurator.getConfiguration()
configuration     = configurator.getScreenConfiguration(all_configuration, screen_name)

if configuration['computer'] is None:
    sys.exit()

screen        = configuration['screen']
computer      = configuration['computer']
computer_name = configuration['computer']['name']

if options.getEnvironment('blend_file') is not None:
    if computer['system']['anchor'] is not None:
        blend_file = os.path.join(computer['system']['anchor'], options.getEnvironment('blend_file'))
    else:
        blend_file = options.getEnvironment('blend_file')
else:
    blend_file = None

temp_path = computer['system']['temp_path']
if not os.path.isdir(temp_path):
    os.makedirs(temp_path)

pid_file = os.path.join(temp_path, computer_name + '_' + screen_name + '.pid')

log_path = computer['system']['log']['path']
if not os.path.isdir(log_path):
    os.makedirs(log_path)

if options.getEnvironment('log_run'):
    log_out_file = os.path.join(log_path, 'run_' + computer_name + '_' + screen_name + '.out')
    log_err_file = os.path.join(log_path, 'run_' + computer_name + '_' + screen_name + '.err')
else:
    log_out_file = os.devnull
    log_err_file = os.devnull

if options.getEnvironment('status'):
    os.chdir(os.path.dirname(blend_file))

    blender_player = computer['system']['blenderplayer']
    cmd            = [blender_player['executable']]
    if screen['display']['options'] is not None:
        cmd += screen['display']['options'].split(' ')

    cmd += [os.path.basename(blend_file), '-']

    cmd.append('--config=' + options.getEnvironment('config'))
    cmd.append('--screen=' + options.getEnvironment('screen'))
    cmd.append('--config-file='+options.getEnvironment('config_file'))
    cmd.append('--config-paths='+options.getEnvironment('config_paths'))

    if ('environments' in blender_player) and (blender_player['environments'] is not None):
        environ = blender_player['environments']
    else:
        environ = []
    try:
        if screen['display']['environments'] is not None:
            environ.update(screen['display']['environments'])
    except KeyError:
        pass
    if 'PYTHONPATH' in environ:
        environ['PYTHONPATH'] = environ['PYTHONPATH'] + os.pathsep + root
    else:
        environ['PYTHONPATH'] = root

    if sys.platform == "win32":
        import getpass
        environ['SystemRoot'] = os.path.join('C:',os.sep,'Windows')
        environ['USERNAME'] = getpass.getuser()

    if options.getEnvironment('background'):
        cmd.append('-b')
        if computer['system']['log']['clear_previous']:
            from blender_cave import logger
            (log_file, verbosity) = logger.getInformation(configuration)
            try:
                os.remove(log_file)
            except:
                pass

        with open(log_out_file, "w") as log_out:
            with open(log_err_file, "w") as log_err:
                process = subprocess.Popen(cmd, env=environ, stderr=log_err, stdout=log_out)
                file = open(pid_file, 'w')
                file.write(str(process.pid))
                file.close()
    else:
        subprocess.Popen(cmd, env=environ).communicate()[0]

else:
    try:
        file = open(pid_file, 'r')
    except:
        pass
    else:
        pid = file.read(1024)
        file.close()
        try:
            os.kill(int(pid), signal.SIGTERM)
        except OSError:
            pass
        try:
            os.remove(pid_file)
        except:
            pass
